import java.util.ArrayList;

/******************************************************************************
*  Statistical bin for a given group of characters.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-07-01
*  @version  1.02
******************************************************************************/

public class StatBin {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Array of characters included. */
	ArrayList<Character> bin; 

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor.
	*/
	public StatBin () {
		bin = new ArrayList<Character>();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Add a new character into these stats.
	*/
	public void addCharacter (Character c) {
		bin.add(c);
	}

	/**
	*  Get size of the bin.
	*/
	public int size () { 
		return bin.size(); 
	}

	/**
	*  Square an integer.
	*/
	static long square (long i) {
		return i * i;	
	}

	/**
	*  Compute a mean from num and sum.
	*/
	double mean (int num, long sum) {
		return (double) sum/num;	
	}

	/**
	*  Compute a standard deviation from num, sum, and sum-of-squares.
	*/
	double stdev (int num, long sum, long sumSquare) {
		return Math.sqrt((sumSquare - (double) square(sum)/num)/num);
	}

	/**
	*  Generate and print all the stats we have available.
	*/
	public void printAllStats () {
		int size = size();
		System.out.print(size + "\t");
		if (size < 1) {
			System.out.println("No data");
		}
		else {
		
			// Sum hit points, ability scores, ages
			long sumAge = 0;
			long sumHitPoints = 0;
			Ability[] abilityValues = Ability.values();
			long [] sumAbilityScore = new long[Ability.length];
			for (Character c: bin) {
				sumAge += c.getAge();
				sumHitPoints += c.getHitPoints();
				for (Ability a: abilityValues) {
					sumAbilityScore[a.ordinal()] += c.getAbilityScore(a);
				}
			}

			// Print means
			double meanAge = mean(size, sumAge);
			double meanHp = mean(size, sumHitPoints);
			System.out.print(Math.round(meanHp) + "\t");
			for (Ability a: abilityValues) {
				double meanAbility = mean(size, sumAbilityScore[a.ordinal()]);
				System.out.print(Math.round(meanAbility) + "\t");
			}
			System.out.print(Math.round(meanAge) + "\t");
 		}
		System.out.println("");
	}

	/**
	*  Generate and print all individual data we have available.
	*/
	public void printAllData () {
		for (Character c: bin) {
			System.out.print(c.getHitPoints() + "\t");
			for (Ability a: Ability.values()) {
				System.out.print(c.getAbilityScore(a) + "\t");
			}
			System.out.println();
		}
	}
}

